/**********************************************************************
 * Copyright 1998 Sybase, Inc. and its subsidiaries.  All rights
 * reserved.
 **********************************************************************/
 
/**********************************************************************
 *
 * WAXString implementation.
 *
 * This class works by subclassing WString and overriding the 
 * SetDirty and PrepareForRead calls.  The former gets called whenever
 * someone makes a change to the string, allowing us to free any
 * BSTR handle we're holding for the string.  PrepareForRead is called
 * whenever someone is about to read the string, allowing us to
 * convert the BSTR into a normal WString as appropriate.
 *
 * Most of the other functions in this class are redefinitions of
 * WString functions overloaded to take or return a WAXString instead.
 *
 * Author:  Eric Giguere
 * Version: 1.0, 21 Jan 1998
 *
 **********************************************************************/

#if defined(__BUILDING_WCM_DLL__) || defined(__BUILDING_WCM_EXE__)
#include "wxdef.hpp"
#include "wxdata.hpp"
#include "wole_h.hpp"
#include "woleutil.hpp"
#include "wolebstr.hpp"
#else
#include "wpch.hpp"
#endif

#include "WAXString.hpp"

WDefineSubclass( WAXString, WString );

//
// Constructors
//

WAXString::WAXString()
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//*********************
{
    Create();
}

WAXString::WAXString( const WString & str )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//*****************************************
{
    Create( str );
}

WAXString::WAXString( const WBString & str )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//******************************************
{
    Create( str );
}

WAXString::WAXString( const WChar *str, WULong numChars )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//*******************************************************
{
    Create( str, numChars );
}

WAXString::WAXString( WBStr handle, WBool copyHandle, WBool deleteHandle )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//************************************************************************
{
    Create( handle, copyHandle, deleteHandle );
}

WAXString::WAXString( const WResourceID & id, WModuleHandle modHandle )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//**********************************************************************
{
    Create( id, modHandle );
}

WAXString::WAXString( const WMessageID & id, WModuleHandle modHandle )
: WString()
, _handle( NULL )
, _deleteHandle( TRUE )
//********************************************************************
{
    Create( id, modHandle );
}

//
// Create a null string
//

WBool WAXString::Create()
//***********************
{
    Clear();
    
    _stringData = WString::_NotYetLoaded;
    
    return TRUE;
}

//
// Create a string from another WString/WAXString.
//

WBool WAXString::Create( const WString & str )
//********************************************
{
    return WString::Create( str );
}

//
// Create a string from raw text.
//

WBool WAXString::Create( const WChar *str, WULong numChars )
//**********************************************************
{
    return WString::Create( str, numChars );
}

//
// Create a string from a WBString.
//

WBool WAXString::Create( const WBString & str )
//*********************************************
{
    return Create( str.GetHandle(), TRUE, TRUE );
}

//
// Create a string from a BSTR.
//

WBool WAXString::Create( WBStr handle, WBool copyHandle, WBool deleteHandle
)
//**************************************************************************

*
{
    Clear();
    
    if( copyHandle && handle ){
	    handle = ::SysAllocStringLen( handle, ::SysStringLen( handle ) );
    }

    _handle       = handle;
    _deleteHandle = deleteHandle;
    _stringData   = WString::_NotYetLoaded;

    return TRUE;
}

//
// Create a string from a resource ID.
//

WBool WAXString::Create( const WResourceID & id, WModuleHandle modHandle )
//************************************************************************
{
    return WString::Create( id, modHandle );
}

//
// Create a string from a message ID.
//

WBool WAXString::Create( const WMessageID & id, WModuleHandle modHandle )
//***********************************************************************
{
    return WString::Create( id, modHandle );
}

//
// Destructor
//

WAXString::~WAXString()
//*********************
{
    // Superclass will call Clear, which will
    // call SetDirty, which will do the right thing.
}

//
// SetDirty
//

WBool WAXString::SetDirty( WBool dirty )
//**************************************
{
    if( dirty ){
        if( _handle && _deleteHandle ){
            ::SysFreeString( _handle );
        }

        _handle       = NULL;
        _deleteHandle = TRUE;
    }

    return TRUE;
}

//
// Called every time someone is about to read
// the string, so we can actually go and create
// the thing....
//

WBool WAXString::PrepareForRead() const
//*************************************
{
     if( _stringData == WString::_NotYetLoaded ){
	     WAXString *self = const_cast<WAXString *>( this );

	     BSTR  oldHandle = _handle;
	     WBool oldDelete = _deleteHandle;
    
	     self->_handle       = NULL;
	     self->_deleteHandle = FALSE;

         if( oldHandle ){
	         self->CreateUnicode( oldHandle, ::SysStringLen( oldHandle ) );
	     } else {
	         self->CreateUnicode( L"" );
	     }

	     self->_handle       = oldHandle;
	     self->_deleteHandle = oldDelete;
     }

     return TRUE;
}

//
// Returns a BSTR.
//

WBStr WAXString::GetHandle( WBool copyHandle ) const
//**************************************************
{
    if( !_handle && _stringData != WString::_NotYetLoaded ){
	     WAXString *self = const_cast<WAXString *>( this );

    	 self->_handle       = ::SysAllocStringLen( GetUnicodeText(),
	                                            GetUnicodeLength() );
	     self->_deleteHandle = TRUE;
    }
    
    return ( copyHandle ? Copy( _handle ) : _handle );
}

//
// Copies a BSTR.
//

static WBStr WAXString::Copy( WBStr handle )
//******************************************
{
    WBStr newHandle = NULL;
    
    if( handle ){
        newHandle = ::SysAllocStringLen( NULL, ::SysStringLen( handle ) );
        if( newHandle ){
            memcpy( newHandle, handle, ::SysStringByteLen( handle ) );
        }
    }
    
    return newHandle;
}

//
// Sets a new BSTR.
//

WBool WAXString::SetHandle( WBStr handle, WBool copyHandle,
                            WBool deleteHandle )
//*********************************************************
{
    return Create( handle, copyHandle, deleteHandle );
}

//
// = operator
//

WAXString & WAXString::operator=( const WString & str )
//*****************************************************
{
    Create( str );
    return *this;
}

WAXString & WAXString::operator=( const WUnicodeChar *str )
//*********************************************************
{
    CreateUnicode( str );
    return *this;
}

WAXString & WAXString::operator=( const WAnsiChar *str )
//******************************************************
{
    CreateAnsi( str );
    return *this;
}

WAXString & WAXString::operator=( const WBString & str )
//******************************************************
{
    Create( str );
    return *this;
}

//
// += operator
//

WAXString & WAXString::operator+=( const WString & str )
//******************************************************
{
    Concat( str );
    return *this;
}

WAXString & WAXString::operator+=( const WBString & str )
//*******************************************************
{
    Concat( (WString) str );
    return *this;
}

WAXString & WAXString::operator+=( const WUnicodeChar *str )
//**********************************************************
{
    Concat( str );
    return *this;
}

WAXString & WAXString::operator+=( const WAnsiChar *str )
//*******************************************************
{
    Concat( str );
    return *this;
}

WAXString & WAXString::operator+=( WUnicodeChar u )
//*************************************************
{
    Concat( u );
    return *this;
}

WAXString & WAXString::operator+=( WAnsiChar a )
//**********************************************
{
    Concat( a );
    return *this;
}

//
// + operator
//

WAXString WAXString::operator+( const WString & str )
//***************************************************
{
    WAXString a( *this );
    a.Concat( str );
    return a;
}

WAXString WAXString::operator+( const WBString & str )
//****************************************************
{
    WAXString a( *this );
    a.Concat( (WString) str );
    return a;
}

WAXString WAXString::operator+( const WUnicodeChar *str )
//*******************************************************
{
    WAXString a( *this );
    a.Concat( str );
    return a;
}

WAXString WAXString::operator+( const WAnsiChar *str )
//****************************************************
{
    WAXString a( *this );
    a.Concat( str );
    return a;
}

WAXString WAXString::operator+( WUnicodeChar u )
//**********************************************
{
    WAXString a( *this );
    a.Concat( u );
    return a;
}

WAXString WAXString::operator+( WAnsiChar a )
//*******************************************
{
    WAXString b( *this );
    b.Concat( a );
    return b;
}

WAXString WEXPORT operator+( const WAnsiChar *a, const WAXString & b )
//********************************************************************
{
    WAXString c;
    
    c.CreateAnsi( a );
    c.Concat( b );
    
    return c;
}

WAXString WEXPORT operator+( const WUnicodeChar *a, const WAXString & b )
//***********************************************************************
{
    WAXString c;
    
    c.CreateUnicode( a );
    c.Concat( b );
    
    return c;
}

//
// Left
//

WAXString WAXString::Left( WULong numChars ) const
//************************************************
{
    return WAXString( WString::Left( numChars ) );
}

//
// Right
//

WAXString WAXString::Right( WULong numChars ) const
//*************************************************
{
    return WAXString( WString::Right( numChars ) );
}

//
// Strip
//

WAXString WAXString::Strip( WBool fromBeg, WBool fromEnd ) const
//**************************************************************
{
    return WAXString( WString::Strip( fromBeg, fromEnd ) );
}

//
// Substring
//

WAXString WAXString::Substring( WULong startAt, WULong numChars ) const
//*********************************************************************
{
    return WAXString( WString::Substring( startAt, numChars ) );
}

//
// Free
//

static void WAXString::Free( WBStr handle )
//*****************************************
{
    if( handle != NULL ){
        ::SysFreeString( handle );
    }
}
